/*
 * Common (OS-independent) definitions for
 * Broadcom 802.11abg Networking Device Driver
 *
 * Copyright 2005-2006, Broadcom Corporation
 * All Rights Reserved.                
 * 
 * THIS SOFTWARE IS OFFERED "AS IS", AND BROADCOM GRANTS NO WARRANTIES OF ANY
 * KIND, EXPRESS OR IMPLIED, BY STATUTE, COMMUNICATION OR OTHERWISE. BROADCOM
 * SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS
 * FOR A SPECIFIC PURPOSE OR NONINFRINGEMENT CONCERNING THIS SOFTWARE.
 * $Id$
 */

#ifndef _wlc_h_
#define _wlc_h_

#include <wl_dbg.h>
#include <wlioctl.h>


/* forward declare */

typedef struct wlc_info wlc_info_t;
typedef struct phy_pub wlc_phy_t;
typedef struct led_info led_info_t;
typedef struct apps_wlc_psinfo apps_wlc_psinfo_t;
typedef struct scb_module scb_module_t;
typedef struct wlc_frminfo wlc_frminfo_t;

#define	RXBUFSZ		PKTBUFSZ	

#define MAXJOINPREFS		4	/* max # user-supplied join prefs */
#define MAXWPACFGS		16	/* max # wpa configs */
#define WLC_MAXBSSCFG		1	/* max # BSS configs */
#define MAXMACLIST		MAXSCB	/* max # source MAC matches */
#define MA_WINDOW_SZ		8	/* moving average window size */
#define	HWRXOFF			30	/* chip rx buffer offset */
#define	INVCHANNEL		255	/* invalid channel */
#define	MAXCOREREV		12	/* max # supported core revisions (0 .. MAXCOREREV - 1) */
#define	NSCBHASH		(MAXSCB/8) /* # scb hash buckets */
#define AIDMAPSZ		(ROUNDUP(MAXSCB, NBBY)/NBBY)	/* aid bitmap size in bytes */
#define NTXRATE			64	/* # tx MPDUs rate is reported for */
#define	WLC_MAXFRAG		16	/* max fragments per SDU */
#define WLC_WEP_IV_SIZE		4	/* size of WEP1 and WEP128 IV */
#define WLC_WEP_ICV_SIZE	4	/* size of WEP ICV */
#define WLC_BCN_TIMEOUT		8	/* seconds w/o bcns until loss of connection */
#define WLC_FREQTRACK_THRESHOLD	5	/* seconds w/o beacons before we increase the
					 * frequency tracking bandwidth
					 */
#define WLC_FREQTRACK_DETECT_TIME	2	/* seconds in which we must detect
						 * beacons after setting the wide
						 * bandwidth
						 */
#define WLC_FREQTRACK_MIN_ATTEMPTS	3	/* minimum number of times we should
						 * try to get beacons using the wide freq
						 * tracking b/w
						 */
#define WLC_FREQTRACK_TIMEOUT	30	/* give up on wideband frequency tracking after
					 * 30 seconds
					 */
#define WLC_MAX_TBTT_OFFSET	20	/* maximum tolerable offset of beacon TS from TBTT in ms */
#define WECA_ASSOC_TIMEOUT	300	/* authentication or association timeout in ms */
#define WLC_MAXMODULES		10	/* max #  wlc_module_register() calls */

/* freqtrack_override values */
typedef enum _freqtrack_override_t {
	FREQTRACK_AUTO = 0,
	FREQTRACK_ON,
	FREQTRACK_OFF
}freqtrack_override_t;

#define NUM_UCODE_FRAMES        4	/* Num ucode frames for TSSI estimation */


#define	MAXCORES		2 /* Maximum cores for rst array */
#define	MAXBANDS		2 /* Maximum #of bands */

#define WLC_CHANNEL_QA_NSAMP	2	/* size of channel_qa_sample array */
#define WLC_RM_RPI_INTERVAL	20	/* ms, time between RPI/Noise samples */

#define	WLC_RATEPROBE_RATE	(6 * 2)	/* 6Mbps ofdm rateprobe rate */

/* Maximum wait time for a MAC suspend */
#define	WLC_MAX_MAC_SUSPEND	35000	/* uS: 35mS is max packet time (for largest MTU pkt) */

/* Probe Response timeout - responses for probe requests older that this are tossed, zero to disable
 */
#define WLC_PRB_RESP_TIMEOUT	0 /* Disable probe repsonse timeout */

#define WLC_VLAN_TAG_LEN	4	/* Duplicated to avoid including <proto/vlan.h> */

/* transmit buffer max headroom for protocol headers */
#define TXOFF (D11_TXH_LEN + D11_PHY_HDR_LEN + DOT11_A4_HDR_LEN + DOT11_QOS_LEN + \
	DOT11_IV_MAX_LEN + DOT11_LLC_SNAP_HDR_LEN + WLC_VLAN_TAG_LEN)

struct wlcband;				/* forward declaration */


/* For managing scan result lists */
typedef struct {
	uint		count;
	wlc_bss_info_t*	ptrs[MAXBSS];

} wlc_bss_list_t;

/* wlc_bss_info flag bit values */
#define WLC_BSS_54G		0x01	/* BSS is a legacy 54g BSS */
#define WLC_BSS_RSSI_ON_CHANNEL	0x02	/* RSSI measurement was from the same channel as BSS */
#define WLC_BSS_WME		0x04	/* BSS is WME capable */
#define WLC_BSS_BRCM		0x08	/* BSS is BRCM */
#define WLC_BSS_WPA		0x10	/* BSS is WPA capable */

/* pkt type used to indicate a roaming STA */
#define ROAM_PKT_TYPE	0x806	/* currently arp */
#define ARP_CMD_OFF		6 /* Unused */
#define ARP_RESP		2 /* Unused */

/* wlc_BSSinit actions */
#define WLC_BSS_JOIN		0 /* join action */
#define WLC_BSS_START		1 /* start action */

/* scan times in milliseconds */
#define WLC_SCAN_MIN_PROBE_TIME	10	/* minimum useful time for an active probe */
#define WLC_SCAN_HOME_TIME	45	/* time for home channel processing */
#define WLC_SCAN_ASSOC_TIME 20 /* time to listen on a channel for probe resp while associated */
#define WLC_SCAN_UNASSOC_TIME 40 /* time to listen on a channel for probe resp while unassociated */
#define WLC_SCAN_PASSIVE_TIME 110 /* time to listen on a channel for beacons for passive scanning */
#define WLC_SCAN_NPROBES	2	/* do this many probes on each channel for an active scan */
#define WLC_SCAN_AWAY_LIMIT	100	/* max time to be away from home channel before returning */
#define WLC_SCAN_PS_PREP_TIME	10	/* average time to announce PS mode */
#define WLC_SCAN_PS_OVERHEAD	50	/* overhead time to allow for one long PS announce delay */

#define WLC_STA_RETRY_MAX_TIME	3600	/* maximum config value for sta_retry_time (1 hour) */

/* scan_pass state values */
#define WLC_SCAN_ABORT		-2 /* Abort the scan */
#define WLC_SCAN_START		-1 /* Start the scan */
#define WLC_SCAN_CHANNEL_PREP	0  /* Prepare the channel for the scan */

/* scan state bits */
#define SCAN_STATE_SUPPRESS	(1 << 0)
#define SCAN_STATE_SAVE_PRB	(1 << 1)
#define SCAN_STATE_PASSIVE	(1 << 2)
#define SCAN_STATE_WSUSPEND	(1 << 3)
#define SCAN_STATE_RADAR_CLEAR	(1 << 4)
#define SCAN_STATE_PSPEND	(1 << 5)
#define SCAN_STATE_SUSPEND_REQ	(1 << 6)

#define WLC_2050_ROAM_TRIGGER	(-70) /* Roam Trigger for 2050 */
#define WLC_2050_ROAM_DELTA	(20)  /* Roam Delta for 2050 */
#define WLC_2050_ENCORE_ROAM_TRIGGER (-75) /* Roam Trigger for 2050-51 radio with encore chip */
#define WLC_2051_ROAM_TRIGGER	(-70) /* Roam Trigger for 2051 */
#define WLC_2051_ROAM_DELTA	(20)  /* Roam Delta for 2051 */
#define WLC_2053_ROAM_TRIGGER	(-60) /* Roam Trigger for 2053 */
#define WLC_2053_ROAM_DELTA	(15)  /* Roam Delta for 2053 */
#define WLC_2060WW_ROAM_TRIGGER	(-75) /* Roam Trigger for 2060ww */
#define WLC_2060WW_ROAM_DELTA	(15)  /* Roam Delta for 2060ww */
#define WLC_ROAM_SCAN_PERIOD	(10)  /* Scan period for roaming */

/* Count of packet callback structures. Set to the number of SCBs since a STA
 * can queue up a rate callback for each IBSS STA it knows about, and an AP can
 * queue up an "are you there?" Null Data callback for each associated STA
 */
#define	MAX_PKT_CB		MAXSCB

/* Rx Antenna diversity control values */
#define	ANTDIV_FORCE_0		0	/* Use antenna 0 */
#define	ANTDIV_FORCE_1		1	/* Use antenna 1 */
#define	ANTDIV_START_1		2	/* Choose starting with 1 */
#define	ANTDIV_START_0		3	/* Choose starting with 0 */
#define	ANTDIV_ENABLE		3	/* APHY bbConfig Enable RX Diversity */

/* Tx Antenna control values */
#define TXANT_0			0	/* Tx on antenna 1, "Main" */
#define TXANT_1			1	/* Tx on antenna 1, "Aux" */
#define TXANT_LAST_RX		3	/* Tx on phy's last good Rx antenna */
#define TXANT_DEF		3	/* driver's default tx antenna setting */

#define WLC_MIN_CNTRY_ELT_SZ	6	/* Min size for 802.11d Country Info Element. */

#if CONF_MSK(D11CONF, 0xb) || CONF_GE(D11CONF, MAXCOREREV)
#error "Configuration for D11CONF includes unsupported versions."
#endif /* Bad versions */

#define	VALID_COREREV(corerev)	CONF_HAS(D11CONF, corerev)

/* values for shortslot_override */
#define WLC_SHORTSLOT_AUTO	-1 /* Driver will manage Shortslot setting */
#define WLC_SHORTSLOT_OFF	0  /* Turn off short slot */
#define WLC_SHORTSLOT_ON	1  /* Turn on short slot */

/* values for barker_preamble */
#define WLC_BARKER_SHORT_ALLOWED	0 /* Short pre-amble allowed */
#define WLC_BARKER_LONG_ONLY		1 /* No short pre-amble allowed */

#define	WLC_IBSS_BCN_TIMEOUT	4 /* Timeout to indicate that various types of IBSS beacons have
				   * gone away
				   */


#define	EHINIT(eh, src, dst, type) \
	bcopy((uchar*)dst, (eh)->ether_dhost, ETHER_ADDR_LEN); \
	bcopy((uchar*)src, (eh)->ether_shost, ETHER_ADDR_LEN); \
	(eh)->ether_type = hton16(type);

/* assoc_state values */
#define AS_IDLE			0 /* Idle state */
#define AS_SCAN			1 /* Scan state */
#define AS_SENT_AUTH_1		2 /* Sent Authentication 1 state */
#define AS_SENT_AUTH_2		3 /* Sent Authentication 2 state */
#define AS_SENT_AUTH_3		4 /* Sent Authentication 3 state */
#define AS_SENT_ASSOC		5 /* Sent Association state */
#define AS_WAIT_RCV_BCN		6 /* Waiting to receive beacon state */
#define AS_SYNC_RCV_BCN		7 /* Waiting Re-sync beacon state */
#define AS_WAIT_DISASSOC	8 /* Waiting to disassociate state */

/* assoc_type values */
#define AS_ASSOCIATION		1 /* Associated */
#define AS_ROAM			2 /* Roaming association */

/* A fifo is full. Clear precedences related to that FIFO */
#define WLC_TX_FIFO_CLEAR(wlc, fifo) ((wlc)->tx_prec_map &= ~(wlc)->fifo2prec_map[fifo])

/* Fifo is NOT full. Enable precedences for that FIFO */
#define WLC_TX_FIFO_ENAB(wlc, fifo)  ((wlc)->tx_prec_map |= (wlc)->fifo2prec_map[fifo])

/* TxFrameID */
/* seq and frag bits: SEQNUM_SHIFT, FRAGNUM_MASK (802.11.h) */
/* rate epoch bits: TXFID_RATE_SHIFT, TXFID_RATE_MASK ((wlc_rate.c) */
#define TXFID_QUEUE_MASK	0x0007 /* Bits 0-2 */
#define TXFID_SEQ_MASK		0xFFE0 /* Bits 5-15 */
#define TXFID_SEQ_SHIFT		5      /* Number of bit shifts */

/* promote boardrev */
#define BOARDREV_PROMOTABLE	0xFF	/* from */
#define BOARDREV_PROMOTED	1	/* to */

/* power-save mode definitions */
#define PSQ_PKTS_BCMC		50	/* max # of enq'd PS bc/mc pkts */
#define PSQ_PKTS_LO		5	/* max # PS pkts scb can enq */
#define	PSQ_PKTS_HI		125	

/* if wpa is in use then portopen is true when the group key is plumbed otherwise it is always true
 */
#define WLC_PORTOPEN(wlc) ((wlc->cfg.WPA_auth != WPA_AUTH_DISABLED) ? wlc->wsec_portopen : TRUE)
/* conditions under which the PM bit should be set in outgoing frames and the WAKE bit is meaningful
 */
#define PS_ALLOWED(wlc) (wlc->PMenabled && !wlc->PM_override && wlc->pub.BSS  && \
	wlc->pub.associated && WLC_PORTOPEN(wlc) && wlc->dtim_programmed)

/* 802.1D Priority to TX FIFO number for wme */
extern uint8 wme_prio2fifo[];

/* rm request types */
#define WLC_RM_TYPE_NONE		0 /* No Radio measurement type */
#define WLC_RM_TYPE_BASIC		1 /* Basic Radio measurement type */
#define WLC_RM_TYPE_CCA			2 /* CCA Radio measurement type */
#define WLC_RM_TYPE_RPI			3 /* RPI Radio measurement type */

/* rm request flags */
#define WLC_RM_FLAG_PARALLEL		(1<<0) /* Flag for setting for Parallel bit */
#define WLC_RM_FLAG_LATE		(1<<1) /* Flag for LATE bit */
#define WLC_RM_FLAG_INCAPABLE		(1<<2) /* Flag for Incapable bit */
#define WLC_RM_FLAG_REFUSED		(1<<3) /* Flag for REFUSED */

/* rm report type */
#define WLC_RM_CLASS_NONE		0 /* No report */
#define WLC_RM_CLASS_IOCTL		1 /* IOCTL type of report */
#define WLC_RM_CLASS_11H		2 /* Report for 11H */

/*
 * Detect Card removed.
 * Even checking an sbconfig register read will not false trigger when the core is in reset.
 * it breaks CF address mechanism. Accessing gphy phyversion will cause SB error if aphy
 * is in reset on 4306B0-DB. Need a simple accessible reg with fixed 0/1 pattern
 * (some platforms return all 0).
 * If clocks are present, call the sb routine which will figure out if the device is removed.
 */
#define	DEVICEREMOVED(wlc)	((wlc->core->clk) ?	\
	((R_REG(&wlc->regs->maccontrol) & (MCTL_PSM_JMP_0 | MCTL_IHR_EN)) != MCTL_IHR_EN) : \
	(sb_deviceremoved(wlc->pub.sbh)))

/*
 * Multiband 101:
 *
 * A "band" is a radio frequency/channel range.
 * We support the 802.11b band (2.4Ghz) using the B or G phy and the 2050 radio.
 * We support the 802.11a band (5  Ghz) using the A      phy and the 2060 radio.
 *
 * Some chips and boards support a single band (uniband) and some
 * support both B/G and A bands (multiband).
 *
 * Versions of the dot11 io core <= 4 support a single phytype/band.
 * Versions of the dot11 io core >= 5 support multiple phytypes.
 * The sbtmstatelow.gmode flag is used to switch between phytypes.
 *
 * 4306B0 (corerev4) includes two dot11 cores, one with a Gphy and one with an Aphy.
 * 4306C0 and later chips (>= corerev5) include a single dot11 core having
 * some combination of A, B, and/or G cores .
 * The driver multiband design abstracts the core portion
 * (such as dma engines and pio fifos) from the band portion (phy+radio state).
 *
 * For convenience, band 0 is always the B/G phy, and, if it exists,
 * band 1 is the A phy.
 */


/*
 * core state (mac)
 */
typedef struct wlccore {
	uint		coreunit;		/* # d11 core instance */
	uint		coreidx;		/* # sb enumerated core */
	bool		clk;			/* core is out of reset and has clock */

	/* fifo */
	hnddma_t	*di[NFIFO];		/* hnddma handles, per fifo */
	pio_t		*pio[NFIFO];		/* pio handlers, per fifo */
	uint		*txavail[NFIFO];	/* # tx descriptors available */
	int8		txpktpend[NFIFO];	/* tx admission control */
	uint16		xmtfifo_limit[NFIFO];	/* max tx fifo in 256B for each fifo */

	/* gpio */
	uint32		gpio_led_outen;		/* gpio leds used */

	macstat32_t	macstat_delta;		/* mac hw prev read values */
	uint32		tsf_timerlow;		/* value at last bandswitch (only if ncores > 1) */
} wlccore_t;

/* TX Power indexes */
#define TXP_FIRST_CCK  	0  /* Index for first CCK rate */
#define TXP_LAST_CCK 	3  /* Index for last CCK rate */
#define TXP_FIRST_OFDM	4  /* Index for first OFDM rate */
#define TXP_LAST_OFDM	11 /* Index for last OFDM rate */
#define TXP_NUM_RATES	12 /* number of rates */

/*
 * band state (phy+ana+radio)
 */
typedef struct wlcband {
	wlc_phy_t	*pi;			/* pointer to phy specific information */
	int		bandtype;		/* WLC_BAND_2G, WLC_BAND_5G */
	uint		bandunit;		/* bandstate[] index */
	uint8		gmode;			/* gmode (see wlioctl.h) */
	bool		init;			/* band has been initialized */

	struct scb	*bcmc_scb;		/* permanent mcast ap-ps scb */
	struct scb	*hwrs_scb;		/* permanent scb for hw rateset */

	struct rateset	defrateset;		/* band-specific copy of default_bss.rateset */

	ratespec_t 	rspec_override;		/* 802.11 rate override */
	ratespec_t	mrspec_override;		/* multicast rate override */
	bcm_tlv_t*	sup_chan_ie;		/* Supported Channels Info Elt */
	uint32		mhf;			/* union of ucode M_HOST_FLAG and M_HOST_FLAG2 */
	struct rateset	hw_rateset;		/* rates supported by chip (phy-specific) */
	uint8		basic_rate[WLC_MAXRATE+1]; /* basic rates indexed by rate */
	int8		antgain;		/* antenna gain from srom */
	int		roam_trigger;		/* "good enough" threshold for current AP */
	uint		roam_delta;		/* "signif better" thresh for new AP candidates */
	int		roam_trigger_def;	/* roam trigger init. value */
	uint		roam_delta_def;		/* roam delta init. value */
	uint		phycal_nslope;		/* time of last nrssislope calibration */
	uint		phycal_noffset;		/* time of last nrssioffset calibration */
	uint16		CWmin;	/* The minimum size of contention window, in unit of aSlotTime */
	uint16		CWmax;	/* The maximum size of contention window, in unit of aSlotTime */
} wlcband_t;

/* generic function callback takes just one arg */
typedef void (*cb_fn_t)(void *);

/* tx completion callback takes 3 args */
typedef void (*pkcb_fn_t)(wlc_info_t *wlc, uint txstatus, void *arg);

typedef struct {
	pkcb_fn_t fn;		/* function to call when tx frame completes */
	void *arg;		/* void arg for fn */
	uint8 nextidx;		/* index of next call back if threading */
	bool entered;		/* recursion check */
} pkt_cb_t;

#define WLC_MAX_BRCM_ELT	32	/* Max size of BRCM proprietary elt */

/* Indexes for A band txpwr from srom */
#define A_LOW_CHANS  0  /* Index for low channels in A band */
#define A_MID_CHANS  1  /* Index for mid channels in A band */
#define A_HIGH_CHANS 2  /* Index for high channels in A band */
#define G_ALL_CHANS  0 /* Index for all channels in G band */

/* aci_state state bits */
#define ACI_ACTIVE	(1 << 1)		/* enabled either manually or automatically */
#define ACI_AUTO	(1 << 2)		/* Auto or manual */

/* module control blocks */
typedef	struct {
	char name[32];               /* module name : NULL indicates empty array member */
	const wlc_iovar_t *iovars;	/* iovar table */
	void *hdl;			/* handle passed when handler 'doiovar' is called */
	watchdog_fn_t watchdog_fn;	/* watchdog handler */
	iovar_fn_t iovar_fn;		/* iovar handler */
	down_fn_t down_fn;		/* down handler */
} modulecb_t;

/* Allowed transmit features for scb */
typedef enum scb_txmod {
	SCB_START, /* Node to designate start of the tx path */
	SCB_CRAM,
	SCB_APPS,
	SCB_AMSDU,
	SCB_BA,
	SCB_AMPDU,
	SCB_TRANSMIT,	/* Node to designate enqueue to wlc->txq */
	SCB_LAST
} scb_txmod_t;

/* Function that does transmit packet feature processing. prec is precedence of the packet */
typedef void (*txmod_tx_fn_t)(void *ctx, struct scb *scb, void *pkt, uint prec);

/* Callback for txmod when it gets deactivated by other txmod */
typedef void (*txmod_deactivate_fn_t)(void *ctx, struct scb *scb);

/* Callback for txmod when it gets activated */
typedef void (*txmod_activate_fn_t)(void *ctx, struct scb *scb);

/* Structure to store registered functions for a Txmod */
typedef struct txmod_info {
	txmod_tx_fn_t		tx_fn;			/* Process the packet */
	txmod_deactivate_fn_t	deactivate_notify_fn;	/* Handle the deactivation of the feature */
	txmod_activate_fn_t	activate_notify_fn;	/* Handle the activation of the feature */
	void *ctx; 					/* Opaque handle to be passed */
} txmod_info_t;

/*
 * Principle common (os-independent) software data structure.
 */
struct wlc_info {
	wlc_pub_t	pub;			/* wlc public state (must be first field of wlc) */
	void		*wl;			/* pointer to os-specific private state */

	/* submodule handler */
	led_info_t	*ledh;			/* pointer to led specific information */
	void		*sup;			/* pointer to supplicant state */

	void		*btparam;		/* bus type specific cookie */
	d11regs_t	*regs;			/* pointer to device registers */
	int		vars_size;		/* size of vars, free vars on detach */
	uint32		btc_mhf;		/* MHF for Blue Tooth Coexistance */
	void		*phy_sh;		/* pointer to shared phy state */

	bool		sbclk;			/* sb has clock */
	bool		plldown;		/* true if we should power off pll in down state */
	bool		forcefastclk;		/* true if the h/w is forcing the use of fast clk */

	/* version info */
	uint16		vendorid;		/* PCI vendor id */
	uint16		deviceid;		/* PCI device id */
	uint		ucode_rev;		/* microcode revision */
	struct ether_addr perm_etheraddr;	/* original sprom local ethernet address */

	/* multiband */
	uint		ncores;			/* # d11 hardware cores */
	wlccore_t	*core;			/* pointer to active io core */
	wlcband_t	*band;			/* pointer to active per-band state */
	wlccore_t	corestate[MAXCORES];	/* per-core state (one per hw core) */
	wlcband_t	bandstate[MAXBANDS];	/* per-band state (one per phy/radio) */
	bool		bandlocked;		/* disable auto multi-band switching */

	/* interrupt */
	uint32		macintstatus;		/* bit channel between isr and dpc */
	uint32		macintmask;		/* sw runtime master macintmask value */
	uint32		defmacintmask;		/* default "on" macintmask value */
	volatile uint32	maccontrol;		/* maccontrol register sw cache */

	/* up and down */
	bool		radio_monitor;		/* radio timer is running */
	bool		down_override;		/* true=down */
	bool		going_down;		/* down path intermediate variable */

	/* timer */
	struct wl_timer *wdtimer;		/* timer for watchdog routine */
	uint		fast_timer;		/* Periodic timeout for 'fast' timer */
	uint		slow_timer;		/* Periodic timeout for 'slow' timer */
	uint		glacial_timer;		/* Periodic timeout for 'glacial' timer */
	uint		now;			/* # elapsed seconds */
	uint		phycal_mlo;		/* last time measurelow calibration was done */
	uint		phycal_txpower;		/* last time txpower calibration was done */

	struct wl_timer *radio_timer;		/* timer for hw radio button monitor routine */
	struct wl_timer *led_timer;		/* periodic led timer */

	/* promiscuous */
	bool		monitor;		/* monitor (MPDU sniffing) mode */
	bool		bcnmisc_ibss;		/* bcns promisc mode override for IBSS */
	bool		bcnmisc_scan;		/* bcns promisc mode override for scan */
	bool		bcnmisc_monitor;	/* bcns promisc mode override for monitor */
	bool		_piomode;		/* true if pio mode */


	bool		channel_qa_active;	/* true if chan qual measurement in progress */
	bool		bcns_lost;		/* indicate if the STA can see the AP */
	bool		war16165;		/* PCI slow clock 16165 war flag */

	bool		check_for_unaligned_tbtt; /* check unaligned tbtt flag */
	bool		PM_override;		/* no power-save flag, override PM(user input) */
	bool		PMenabled;		/* current power-management state (CAM or PS) */
	bool		PMpending;		/* waiting for tx status with PM indicated set */
	bool		PMblocked;		/* block any PSPolling, PS mode being used to buffer
						 * AP traffic
						 */
	bool		WME_PM_blocked;		/* Can STA go to PM when in WME Auto mode */
	bool		dtim_programmed;	/* wait for dtim beacon */
	bool		wake;			/* host-specified PS-mode sleep state */
	bool		wakeforclkctl;		/* force wake for clkctl change */
	bool		wakefortxfifo;		/* force wake for tx fifo access */
	bool		wakeformacsuspend;	/* force wake for mac suspend and wait */
	bool		wme_noack;		/* enable WME no-acknowledge mode */
	ac_bitmap_t	wme_dp;			/* Discard (oldest first) policy per AC */
	bool		wme_apsd;		/* enable Advanced Power Save Delivery */
	bool		wme_apsd_assoc;		/* an associated STA is using APSD */
	int		vlan_mode;		/* OK to use 802.1Q Tags (ON, OFF, AUTO) */
	uint16		tx_prec_map;		/* Precedence map based on HW FIFO space */
	uint16		fifo2prec_map[NFIFO];	/* pointer to fifo2_prec map based on WME */

	bool		_regulatory_domain;	/* 802.11d enabled? */
	bool		tx_suspended;		/* data fifos need to remain suspended */
	bool		wpa_msgs;		/* enable/disable wpa indications via wlc_wpa_msg */

	/* BSS Configurations */
	wlc_bsscfg_t	*bsscfg[WLC_MAXBSSCFG];	/* set of BSS configurations, idx 0 is default and
						 * always valid
						 */
	wlc_bsscfg_t	cfg;			/* the primary bsscfg (can be AP or STA) */
	int		bsscfg_prb_idx;		/* index of bsscfg using ucode probe response */
	int		bsscfg_bcn_idx;		/* index of bsscfg using ucode beacon(designated
						 * bss)
					 */
	uint		aps_associated;		/* count of active (up) AP bsscfgs */
	uint8		block_datafifo;		/* prohibit posting frames to data fifos */
	bool		bcmcfifo_drain;		/* TX_BCMC_FIFO is set to drain */

	uint8	eventing[WL_EVENTING_MASK_LEN];		/* 128-bit mask for MAC event reporting to
							 * outside world
							 */
	uint8	event_msgs[WL_EVENTING_MASK_LEN];	/* 128-bit mask for MAC event reporting via
							 * data packets
							 */

	struct pktq	txq;			/* common tx pkt Q */

	/* scb data */
	scb_module_t	*scbstate;		/* pointer to scb module state */

	uint		seckeys;		/* 58 key table shm address */

	wlc_event_cb_t callback;		/* event callback function */

	/* channel quality measure */
	int		channel_quality;	/* quality metric(0-3) of last measured channel, or
						 * -1 if in progress
						 */
	uint		channel_qa_channel;	/* channel number of channel being evaluated */
	int8		channel_qa_sample[WLC_CHANNEL_QA_NSAMP]; /* rssi samples of background
								  * noise
								  */
	uint		channel_qa_sample_num;	/* count of samples in channel_qa_sample array */

	/* quiet channels currently for radar sensitivity or 11h support */
	chanvec_t	quiet_channels;		/* channels on which we cannot transmit */

	/* roaming */
	uint		roam_metric;		/* eval metric used in roam scan cb */
	uint		time_since_bcn;		/* second count since our last beacon was seen */
	uint32		tsf_h;		/* tracked TSF high bits (to detect retrograde TSF) */
	uint32		tsf_l;		/* tracked TSF low bits (to detect retrograde TSF) */
	int		roam_scan_block;	/* roam scan frequency mitigator */
	int		roam_scan_period;	/* user-specified roam scan period */
	uint		roam_reason;		/* cache so we can report w/WLC_E_ROAM event */

	/* association */
	struct wl_timer *assoc_timer;		/* timer for auth/assoc request timeout */
	uint		assoc_type;		/* roam or association */
	uint		assoc_state;
	uint		sta_retry_time;		/* time to retry on initial assoc failure */

	uint16		assoc_capability;	/* next (re)association request overrides */
	uint16		assoc_listen;
	struct ether_addr assoc_bssid;
	uint8		*assoc_elts;
	uint		assoc_elts_len;

	struct dot11_assoc_req *assoc_req;	/* last (re)association request */
	uint		assoc_req_len;
	bool		assoc_req_is_reassoc;	/* is a reassoc */
	struct dot11_assoc_resp *assoc_resp;	/* last (re)association response */
	uint		assoc_resp_len;

	uint		sup_type;
	bool		sup_auth_pending;	/* flag for auth timeout */

	wlc_bss_list_t	join_targets;
	uint		join_targets_last;	/* index of last target tried (next: --last) */

	/* 802.11h Quiet Period */
	struct {
		int duration; 		/* TU's */
		int period;
		int offset; 		/* TU's offset from TBTT in Count field */
		uint32 _state;
		int countdown;		/* Beacons from now when quiet period will start */
		uint32 start_prep_tsf;	/* When to start preparing to be quiet(ie shutdown FIFOs)
					 */
		uint32 start_tsf;	/* When to start being quiet */
		uint32 end_tsf;	/* When we can start xmiting again */
		/* dual purpose: wait for offset with beacon & wait for duration */
		struct wl_timer *timer;
	} quiet;

	/* 802.11h Channel Switch Announcement */
	struct {
		uint32	secs;		/* seconds until channel switch */
		int8	channel;	/* target channel */
	} channel_sw;

	/* Demodulator frequency tracking */
	bool		freqtrack;		/* Have we increased the frequency
						 * tracking bandwidth of the
						 * demodulator?
						 */
	uint		freqtrack_starttime;	/* Start time(in seconds) of the last
						 * frequency tracking attempt
						 */
	int8		freqtrack_attempts;	/* Number of times we tried to acquire
						 * beacons by increasing the freq
						 * tracking b/w
						 */
	int8		freqtrack_override;	/* Override setting from registry */

	/* scan */
	struct {
		/* scan times are in milliseconds */
		int		bss_type;	/* Scan for Infra, IBSS, or Any */
		int		ssid_len;
		uchar		ssid[DOT11_MAX_SSID_LEN];
		struct ether_addr bssid;
		uint8		channel_list[MAXCHANNEL];	/* list of channels to scan */
		int		channel_num;	/* length of channel_list */
		int		channel_idx;	/* index in channel_list of current channel we are
						 * scanning
						 */
		uint8		channel_return;	/* channel to return to when scan is done */
		uint8		state;		/* scan state bits */
		int8		pass;		/* current scan pass or scan state */
		int8		nprobes;	/* number of probes per channel */
		uint16		home_time;	/* dwell time for the home channel between channel
						 * scans
						 */
		uint16		active_time;	/* dwell time per channel for active scanning */
		uint16		passive_time;	/* dwell time per channel for passive scanning */
		int		away_limit;	/* number of non-home channels to scan before
						 * returning to home
						 */
		int		away_count;	/* number of non-home channels we have scanned
						 * consecutively
						 */
		uint32		start_tsf;	/* TSF read from chip at start of channel scan */
		struct wl_timer *timer;		/* timer for BSS scan operations */
		cb_fn_t		cb;		/* function to call when scan is done */
		void		*cb_arg;	/* arg to cb fn */
		/* scan defaults */
		struct {
			uint16	home_time;	/* dwell time for the home channel between channel
						 * scans
						 */
			uint16	unassoc_time;	/* dwell time per channel when unassociated */
			uint16	assoc_time;	/* dwell time per channel when associated */
			uint16	passive_time;	/* dwell time per channel for passive scanning */
			int8	nprobes;	/* number of probes per channel */
			int8	passive;	/* scan type: 1 -> passive, 0 -> active */
		} defaults;
	} scan;
	wlc_bss_list_t	scan_results;

	bool ibss_allowed;	/* If FALSE, all IBSS will be ignored during a scan
				 * and the driver will not allow the creation of an IBSS network
				 */

	uint		aci_start_time;		/* adjacent channel interference start time */
	int		aci_exit_check_period;

	uint		txpend16165war;		

	uint		dbgsel;			/* maccontrol debug gpio select */


	bool		assocroam;		/* TRUE: roam to preferred assoc band in oid
						 * broadcast scan
						 */

	wlc_bss_list_t	custom_scan_results;	/* results from ioctl scan */


	uint		_spect_management;	/* 802.11h dot11SpectrumManagementRequired value */

	/* Currently, JP-variants are reported througth WLC_GET_COUNTRY_LIST if TRUE */
	bool		country_list_extended;

	uint16		AID;			/* association ID */
	uint16		counter;		/* per-sdu monotonically increasing counter */

	uint		nmac;			/* # of entries on maclist array */
	int		macmode;		/* allow/deny stations on maclist array */
	struct ether_addr	maclist[MAXMACLIST];	/* list of source MAC addrs to match */

	struct ether_addr	BSSID;		/* Filter for receiving bc/mc frames */
	struct ether_addr	prev_BSSID;	/* MAC addr of last associated AP (BSS) */
	struct ether_addr	announce_BSSID;	/* BSSID to announce in beacon or probe */

	wlc_bss_info_t	default_bss;		/* configured BSS parameters */
	wlc_bss_info_t	target_bss;		/* join attempt BSS parameters */

	/* security */
	wsec_key_t*	wsec_keys[WSEC_MAX_KEYS];	/* dynamic key storage */
	wsec_key_t	wsec_def_keys[WLC_DEFAULT_KEYS];	/* fixed default key storage */
	wsec_key_vec_t	wsec_bsupdate;		/* key update flag for band switch that involves a
						 * core switch
						 */
	bool		wsec_portopen;		/* indicates keys are plumbed */
	bool		wsec_swkeys;		/* indicates that all keys should be
						 * treated as sw keys (used for debugging)
						 */

	/* network config */
	bool	shortpreamble;		/* currently operating with CCK ShortPreambles */
	bool	shortslot;		/* currently using 802.11g ShortSlot timing */
	bool	erp_ie_use_protection;	/* currently advertising Use_Protection */
	bool	erp_ie_nonerp;		/* currently advertising NonERP_Present */
	bool	nonerp_assoc;		/* NonERP STAs associated? */
	bool	ofdm_assoc;		/* OFDM STAs associated? */
	bool	nonabcap_assoc;		/* nonAfterburnerCapable STAs associated? */
	bool	longslot_assoc;		/* Long Slot timing only STAs associated? */
	bool	longpre_assoc;		/* bss/ibss: cck long preamble only STAs associated? */
	bool	include_legacy_erp;	/* include Legacy ERP info elt ID 47 as well as
					 * 802.11g ID 42
					 */
	bool	ignore_bcns;		/* override: ignore non shortslot bcns in a G network */
	bool	barker_overlap_control; /* true=be aware of overlapping BSSs for barker */
	bool	g_protection;		/* use g spec protection */
	bool	g_protection_cts;	/* use CTS-to-Self instead of RTS/CTS */
	bool	n_protection;		/* use OFDM protection on MIMO frames */
	bool	interference_mode_crs;	/* aphy crs state for interference mitigation mode */

	int8	barker_preamble;	/* current Barker Preamble Mode */
	int8	shortslot_override;	/* 802.11g ShortSlot override */
	int	nonerp_ibss_timeout;	/* #seconds until nonerp IBSS beacons gone */
	int	nonerp_ovlp_timeout;	/* #seconds until nonerp overlapping BSS bcns gone */
	int	ofdm_ibss_timeout;	/* #seconds until ofdm IBSS beacons gone */
	int	ofdm_ovlp_timeout;	/* #seconds until ofdm overlapping BSS bcns gone */
	int	longpre_detect_timeout;	/* #seconds until long preamble bcns gone */
	int	barker_detect_timeout;	/* #seconds until bcns signaling Barker long
					 * preamble only gone
					 */
	int	protection_detect_timeout; /* #seconds until bcns signaling Use_Protection gone */
	int	n_protection_ibss_detect_timeout; /* #seconds until bcns signaling
						   * Use_OFDM_Protection gone
						   */
	int8	g_protection_control;	/* AP/IBSS control alg for g spec protection */
	int8	g_protection_override;	/* override for use of g spec protection */
	int8	n_protection_control;	/* AP/IBSS control alg for PreN protection */
	int8	n_protection_override;	/* override for use of PreN protection */

	int8	PLCPHdr_override;	/* 802.11b Preamble Type override */
	int8	txant;			/* userTx antenna setting */
	uint16	phytxant;		/* phyTx antenna setting in txheader */

	int	nmode;			/* PreN mode 0 = disabled, 1 = enabled */

	int	interference_mode;	/* enable interference mitigation mode */
	uint	interference_mode_crs_time; /* time at which crs was turned off */
	uint16	crsglitch_prev;		/* crsglitch count at last watchdog */

	/* do not send prb resp if request older than this, 0 = disable */
	uint16		prb_resp_timeout;

	struct rateset	sup_rates_override;	/* use only these rates in 11g supported rates if
						 * specifed
						 */

	/* PHY parameters */
	uint16		fragthresh[NFIFO];	/* per-fifo fragmentation thresholds */
	uint16		RTSThresh;		/* 802.11 dot11RTSThreshold */
	uint16		SRL;			/* 802.11 dot11ShortRetryLimit */
	uint16		LRL;			/* 802.11 dot11LongRetryLimit */
	uint16		SFBL;			/* Short Frame Rate Fallback Limit */
	uint16		LFBL;			/* Long Frame Rate Fallback Limit */

	int		rssi;			/* RSSI moving average */

	int8		phy_noise_list[MAXCHANNEL];	/* noise right after tx */


	char		brcm_ie[WLC_MAX_BRCM_ELT];	/* brcm proprietary element */


	bool		PSpoll;			/* whether there is an outstanding PS-Poll frame */
	uint8		PM;			/* power-management mode (CAM, PS or FASTPS) */
	uint8		PMidlebcns;		/* bcns since last pkt was tx'd or rx'd */
	bool		PMawakebcn;		/* bcn recvd during current waking state */

	/* packet queue */
	uint		qvalid;			/* DirFrmQValid and BcMcFrmQValid */

	pkt_cb_t	pkt_callback[MAX_PKT_CB + 1]; /* tx completion callback handlers */

	uint32		txretried;		/* tx retried number in one msdu */
	uint		txrateidx;		/* index into tx rate circular buffer */
	ratespec_t     	txrate[NTXRATE][2];	/* circular buffer of prev MPDUs tx rates */

	uint8		nickname[32];		/* Set by vx/linux ioctls but currently unused */

	uint8		spect_state; 		/* Keep various 11h states */

	dot11_quiet_t 	quiet_cmd;
	dot11_channel_switch_t 	csa;
	uint8 		pwr_constraint;

	/* WiFi Multimedia Enhancements (WME) */
	ac_bitmap_t	wme_admctl;		/* bit i set if AC i under admission control */
	uint16		wme_txop[AC_COUNT];	/* current txop for each ac */
	wme_param_ie_t	wme_param_ie;		/* WME parameter info element, which on STA
						 * contains parameters in use locally, and on
						 * AP contains parameters advertised to STA
						 * in beacons and assoc responses.
						 */
	uint8		apsd_sta_qosinfo;	/* Application-requested APSD configuration */
	bool		apsd_sta_usp;		/* Unscheduled Service Period in progress on STA */

	wlc_event_t 	*events;		/* ordered list of events to report */
	uint32	lifetime[AC_COUNT];		/* MSDU lifetime per AC in us */

	modulecb_t	modulecb[WLC_MAXMODULES];

	txmod_info_t  txmod_fns[SCB_LAST];
};


/* quiet.state */
#define WAITING_FOR_FLUSH_COMPLETE 	(1 << 0)
#define WAITING_FOR_INTERVAL		(1 << 1)
#define WAITING_FOR_OFFSET		(1 << 2)
#define WAITING_FOR_DURATION		(1 << 3)
#define SILENCE				(1 << 4)

#define QUIET_STATE(wlc)	0

/* spect_state */
#define NEED_TO_UPDATE_BCN	(1 << 0)	/* Need to decrement counter in outgoing beacon */
#define NEED_TO_SWITCH_CHANNEL	(1 << 1)	/* A channel switch is pending */
#define RADAR_SIM		(1 << 2)	/* Simulate radar detection...for testing */

#define FASTPM_IDLEBCNS	2	/* fast pm mode: transition from PS off->on after this many idle * bcn periods */

/* bandstate array indices */
#define BAND_2G_INDEX		0	/* wlc->bandstate[x] index */
#define BAND_5G_INDEX		1	/* wlc->bandstate[x] index */

#define	CHANNEL_BANDUNIT(wlc, ch)	((ch <= WLC_MAX_2G_CHANNEL) ? BAND_2G_INDEX : BAND_5G_INDEX)
#define	OTHERBANDUNIT(wlc)	((uint)((wlc)->band->bandunit? BAND_2G_INDEX : BAND_5G_INDEX))
#define	BAND2CORE(wlc, bandunit) ((uint)(((wlc)->ncores > 1)? bandunit: 0))

#define IS_MBAND_UNLOCKED(wlc) \
	((NBANDS(wlc) > 1) && !(wlc)->bandlocked)

/* increment counter */
#define WLCNINC(cntr)
#define WLCNINCN(cntr)

#define AID2PVBMAP(aid)	(aid & 0x3fff)
#define AID2AIDMAP(aid)	((aid & 0x3fff) - 1)
/* set the 2 MSBs of the AID */
#define AIDMAP2AID(pos)	((pos + 1) | 0xc000)

/* sum the individual fifo tx pending packet counts */
#define	TXPKTPENDTOT(wlc) (wlc->core->txpktpend[0] + wlc->core->txpktpend[1] + \
	wlc->core->txpktpend[2] + wlc->core->txpktpend[3])

#define WLC_IS_MATCH_SSID(wlc, ssid1, ssid2, len1, len2) \
	((len1 == len2) && !bcmp(ssid1, ssid2, len1))

extern void wlc_link(wlc_info_t *wlc, bool up, struct ether_addr *addr, wlc_bsscfg_t *bsscfg);
extern int wlc_scan_request(wlc_info_t *wlc,
	int bss_type, const struct ether_addr* bssid, const uchar ssid[], int ssid_len,
	int scan_type, int nprobes, int active_time, int passive_time, int home_time,
	const uint16* channel_list, int channel_num, bool save_prb, cb_fn_t fn, void* arg);
extern void wlc_set_ssid(wlc_info_t *wlc, uchar SSID[], int len);

/* helper functions */
extern bool wlc_rateset_isofdm(uint count, uint8 *rates);
extern void wlc_remove_all_keys(wlc_info_t *wlc);
extern int wlc_format_ssid(char* buf, const uchar ssid[], int ssid_len);
extern bool wlc_bssid_is_current(wlc_info_t *wlc, struct ether_addr *bssid);
extern uint32 wlc_mhf(wlc_info_t *wlc, uint32 mask, uint32 val, bool allbands);
extern void wlc_txstatusrecv(wlc_info_t *wlc);
extern void wlc_bss_list_free(wlc_info_t *wlc, wlc_bss_list_t *bss_list);
extern void wlc_update_txpwr_shmem(wlc_pub_t *pub);

void wlc_rateprobe(wlc_info_t *wlc, struct ether_addr *ea,
                   struct scb *scb, ratespec_t rate_override);
void *wlc_senddeauth(wlc_info_t *wlc, struct ether_addr *da, struct ether_addr *bssid,
                      struct scb *scb, uint16 reason_code);
void wlc_mac_event(wlc_info_t* wlc, uint msg, const struct ether_addr* addr,
                   uint result, uint status, uint auth_type, void *data, int datalen);

void wlc_bss_up(wlc_info_t *wlc, wlc_bsscfg_t *cfg);

int wlc_disassociate_client(wlc_info_t *wlc, bool send_disassociate, pkcb_fn_t fn, void *arg);
void wlc_assoc_abort(wlc_info_t *wlc);

extern int wlc_pkt_callback_register(wlc_info_t *wlc, pkcb_fn_t fn, void *arg, void *pkt);
extern void *wlc_sendnulldata(wlc_info_t *wlc, struct ether_addr *da, uint rate_override,
                              uint32 pktflags, int prio);
extern int wlc_wsec(wlc_info_t *wlc, wlc_bsscfg_t *bsscfg, uint32 val);
extern void wlc_read_tsf(wlc_info_t* wlc, uint32* tsf_l_ptr, uint32* tsf_h_ptr);
extern bcm_tlv_t *wlc_find_vendor_ie(uint8 *tlvs, int tlvs_len, uint8 *voui, uint8 *type,
	int type_len);
extern void wlc_set_cwmin(wlc_info_t *wlc, uint16 newmin);
extern void wlc_set_cwmax(wlc_info_t *wlc, uint16 newmax);
extern void wlc_mac_bcn_promisc(wlc_info_t *wlc);
extern void wlc_mac_promisc(wlc_info_t *wlc);
extern void wlc_sendprobe(wlc_info_t *wlc, uchar ssid[], int ssid_len);
extern void wlc_bss_list_xfer(wlc_bss_list_t *from, wlc_bss_list_t *to);
extern void wlc_send_q(wlc_info_t *wlc, struct pktq *q);
extern int wlc_prep_sdu(wlc_info_t *wlc, void **sdu, int *nsdu, uint *fifo);
extern int wlc_prep_pdu(wlc_info_t *wlc, void *pdu, uint *fifo);
extern void wlc_send_action_err(wlc_info_t *wlc, struct dot11_management_header *hdr,
	uint8 *body, int body_len);
extern void *wlc_frame_get_mgmt(wlc_info_t *wlc, uint16 fc, const struct ether_addr *da,
        const struct ether_addr *bssid, uint body_len, uint8 **pbody);
extern void *wlc_frame_get_ctl(wlc_info_t *wlc, uint len);
extern bool wlc_sendmgmt(wlc_info_t *wlc, void *p, struct scb *scb, int preamble,
	wsec_key_t *key, ratespec_t rate_or);
extern bool wlc_sendctl(wlc_info_t *wlc, void *p, struct scb *scb, uint fifo,
	ratespec_t rate_or, bool enq_only);
extern void wlc_recvdata_ordered(wlc_info_t *wlc, struct scb *scb, struct wlc_frminfo *f);

extern void wlc_set_pmstate(wlc_info_t *wlc, bool state);
extern void wlc_uint64_add(uint32* high, uint32* low, uint32 inc_high, uint32 inc_low);
extern void wlc_uint64_sub(uint32* a_high, uint32* a_low, uint32 b_high, uint32 b_low);
extern bool wlc_uint64_lt(uint32 a_high, uint32 a_low, uint32 b_high, uint32 b_low);
extern void wlc_rm_meas_end(wlc_info_t* wlc);
extern void wlc_rm_start(wlc_info_t* wlc);
extern int wlc_rm_abort(wlc_info_t* wlc);
extern uint32 wlc_calc_tbtt_offset(uint32 bi, uint32 tsf_h, uint32 tsf_l);
extern int wlc_roam_scan(wlc_info_t *wlc, uint reason);

/* Shared memory and template access */
extern void wlc_write_shm(wlc_info_t *wlc, uint offset, uint16 v);
extern uint16 wlc_read_shm(wlc_info_t *wlc, uint offset);
extern void wlc_write_template_ram(wlc_info_t *wlc, int offset, int len, void *buf);
extern void wlc_set_shm(wlc_info_t *wlc, uint offset, uint16 v, int len);
extern void wlc_copyto_shm(wlc_info_t *wlc, uint offset, const void* buf, int len);
extern void wlc_copyfrom_shm(wlc_info_t *wlc, uint offset, void* buf, int len);
extern void wlc_set_rcmta(wlc_info_t *wlc, int idx, const struct ether_addr *addr);
extern void wlc_set_addrmatch(wlc_info_t *wlc, int match_reg_offset, const struct ether_addr *addr);

extern void wlc_assoc_change_state(wlc_info_t *wlc, uint newstate);
extern void wlc_txfifo(wlc_info_t *wlc, uint fifo, void *p, bool commit);
extern bool wlc_validate_chip_access(wlc_info_t *wlc);

extern void wlc_update_beacon(wlc_info_t *wlc);
extern void wlc_read_pmq_ea(wlc_info_t *wlc,  struct ether_addr* eaddr);

extern bool wlc_prec_enq(wlc_info_t *wlc, struct pktq *q, void *pkt, int prec);
extern void wlc_pkt_flush(wlc_info_t *wlc, void *pkt, uint16 txstatus);
extern void wlc_pkt_callback_append(wlc_info_t *wlc, void *new_pkt, void *pkt);

extern char* wlc_get_macaddr(wlc_info_t *wlc);
extern uint8* wlc_write_info_elt(uint8 *cp, int id, int len, const void* data);
extern void wlc_coredisable(wlc_info_t* wlc);
extern void wlc_clkctl_clk(wlc_info_t *wlc, uint mode);
extern void wlc_xtal(wlc_info_t* wlc, bool want);
extern void wlc_radio_hwdisable_upd(wlc_info_t* wlc);
extern void wlc_setxband(wlc_info_t *wlc, uint bandunit);
extern void wlc_setband(wlc_info_t *wlc, uint bandunit);
extern void wlc_rateset_elts(wlc_info_t *wlc, uint count, const uint8* rates,
                             struct rateset *sup_rates, struct rateset *ext_rates);
extern void wlc_compute_plcp(wlc_info_t *wlc, ratespec_t rate, uint length, uint8 *plcp);
extern uint wlc_calc_frame_time(wlc_info_t *wlc, ratespec_t ratespec,
	int short_preamble, uint mac_len);
extern void wlc_processpmq(wlc_info_t *wlc);
extern void wlc_set_ratetable(wlc_info_t *wlc);
extern void wlc_scan_default_channels(wlc_info_t *wlc, uint8 *channel_list, int *pchannel_num);
extern void wlc_default_bss(wlc_info_t *wlc);
extern uint wlc_rx_channel(d11rxhdr_t *rxh);
extern void wlc_set_channel(wlc_info_t *wlc, uint channel);

extern void wlc_info_init(wlc_info_t* wlc, bool wme, int unit);
extern bool wlc_timers_init(wlc_info_t* wlc, int unit);

extern const wlc_iovar_t wlc_iovars[];

extern bool wlc_update_brcm_ie(wlc_info_t *wlc);
extern void wlc_txq_scb_free_notify(void *context, struct scb *remove);
extern void wlc_recvdata(wlc_info_t *wlc, osl_t *osh, d11rxhdr_t *rxh, void *p);
extern void wlc_recvdata_sendup(wlc_info_t *wlc, struct scb *scb, bool wds, struct ether_addr *da,
                                void *p);
extern bool wlc_sendsdu(wlc_info_t *wlc, void *sdu);

extern struct dot11_header *wlc_80211hdr(wlc_info_t *wlc, osl_t *osh, void *p,
                                         wlc_bsscfg_t *bsscfg, struct scb *scb,
                                         bool MoreFrag, wsec_key_t *key,
                                         uint8 prio);
extern void wlc_ether_8023hdr(wlc_info_t *wlc, osl_t *osh, struct ether_header *eh, void *p);
extern void wlc_8023_etherhdr(wlc_info_t *wlc, osl_t *osh, void *p);
extern void* wlc_hdr_proc(wlc_info_t *wlc, void *sdu, struct scb *scb);

extern uint16 wlc_sdu_etype(wlc_info_t *wlc, void *sdu);
extern uint8* wlc_sdu_data(wlc_info_t *wlc, void *sdu);
extern uint16 wlc_rcvfifo_limit_get(wlc_info_t *wlc);


extern void wlc_dump_txdesc(d11txh_t *txh);
extern void wlc_dump_txstatus(tx_status_t* txs);

extern void wlc_btc_gpio_disable(wlc_info_t *wlc);

/* stuff pulled in from wlc.c */

/* Interrupt bit error summary.  Don't include I_RU: we refill DMA at other
 * times; and if we run out, constant I_RU interrupts may cause lockup.  We
 * will still get error counts from rx0ovfl.
 */
#define	I_ERRORS	(I_PC | I_PD | I_DE | I_RO | I_XU)
/* default software intmasks */
#define	DEF_RXINTMASK	(I_RI)	/* enable rx int on rxfifo only */
#define	DEF_MACINTMASK	(MI_TXSTOP | MI_TBTT | MI_ATIMWINEND | MI_PMQ | \
			 MI_PHYTXERR | MI_DMAINT | MI_TFS | MI_BG_NOISE | \
			 MI_CCA | MI_TO | MI_GP0 | MI_RFDISABLE)

#define	RETRY_SHORT_DEF			7	/* Default Short retry Limit */
#define	RETRY_SHORT_MAX			255	/* Maximum Short retry Limit */
#define	RETRY_LONG_DEF			4	/* Default Long retry count */
#define	RETRY_SHORT_FB			3	/* Short retry count for fallback rate */
#define	RETRY_LONG_FB			2	/* Long retry count for fallback rate */

#define	MAXTXPKTS		6		/* max # pkts pending */

#define	PIO_TX	1 /* Xmit engine for PIO Fifo */
#define	PIO_RX	2 /* Recv. engine for PIO Fifo */

#define PIOREG(wlc, txrx, fifonum)	(D11REV_LT(wlc->pub.corerev, 8) ? \
	((txrx == PIO_TX) ?     (void *)&(wlc->regs->fifo.f32regs.pioregs[fifonum].b2.tx) : \
				(void *)&(wlc->regs->fifo.f32regs.pioregs[fifonum].b2.rx)) : \
	((txrx == PIO_TX) ?     (void *)&(wlc->regs->fifo.f32regs.pioregs[fifonum].b4.tx) : \
				(void *)&(wlc->regs->fifo.f32regs.pioregs[fifonum].b4.rx)))

extern void wlc_txmod_fn_register(wlc_info_t *wlc, scb_txmod_t feature_id, txmod_tx_fn_t tx_fn,
                                  void *ctx, txmod_deactivate_fn_t deactivate_fn,
                                  txmod_activate_fn_t activate_fn);
extern void wlc_txq_scb_init(void *ctx, struct scb *scb);

#endif	/* _wlc_h_ */
